@@ -1411,7 +1411,7 @@ class MyQtApp(PFAS_SAT_ui.Ui_PFAS_SAT, QtWidgets.QMainWindow):
         self.References = Reference_ui.Ui_References()
         self.References.setupUi(Dialog)
         
-        self.RefDF = pd.read_csv(Path(__file__).parent.parent/'InputData/Data/References.csv')
+        self.RefDF = pd.read_csv(PFAS_SAT_InputData.__path__[0]+'/Data/References.csv')
         references = Table_from_pandas(self.RefDF)
         self.References.TableRef.setModel(references)
         self.References.TableRef.installEventFilter(self)
