@@ -177,13 +177,13 @@ class Project():
                         if pop_up:
                             pop_up('PFAS Flow Warning!', msg, 'Warning')
                         raise ValueError(msg)
-                    elif sum(flw.PFAS.values) > CuttOff:
+                    elif sum(flw.PFAS.values) > CuttOff or sum(flw.PFAS.values) > 1:
                         Inventory.add(prdct, source, prcs, flw)
                         processmodel[ProcessNameRef[prcs]]['Model'].calc(flw)
                         product_2 = processmodel[ProcessNameRef[prcs]]['Model'].products()
                         Project.calc(product_2, prcs, processmodel, FlowParams, Inventory, CuttOff, ProcessNameRef, Treatment_options, pop_up=pop_up)
             else:
-                if sum(product[prdct].PFAS.values) > CuttOff:
+                if sum(product[prdct].PFAS.values) > CuttOff or sum(product[prdct].PFAS.values) > 1:
                     # Change the warning to exception when all the process models are done
                     msg = "No process is defined for {} treatment! \nSelect one of the following processes: \n".format(prdct)
                     for j in Treatment_options[prdct]:
@@ -198,9 +198,10 @@ class Project():
         for p in self.ProcessSet:
             self.ProcessModels[p]['Model'].setup_MC(seed=seed)
 
-    def MC_Next(self):
+    def MC_Next(self, InputFlow):
         NewInputData = []
-        NewInputData += self.InputFlow_object.MC_Next()
+        if InputFlow:
+            NewInputData += self.InputFlow_object.MC_Next()
         for p in self.ProcessSet:
             Raw_newdata = []
             for x in self.ProcessModels[p]['Model'].MC_Next():
@@ -213,24 +214,27 @@ class Project():
         Project.calc(product, 'Start', self.ProcessModels, self.FlowParams, self.Inventory, self.CuttOff, self._ProcessNameRef, self.WasteTreatment)
         return(NewInputData)
 
-    def MC_Run(self, n, TypeOfPFAS='All', signal=None):
+    def MC_Run(self, n, InputFlow=True, TypeOfPFAS='All', signal=None):
         # Print warning only first time
         warnings.simplefilter('once', UserWarning)
 
+        start_time = time.time()
         progress = 0
         progress_steps = [int(x) for x in np.linspace(1, n, 19)]
         MC_results = []
         for i in range(n):
-            NewInputData = self.MC_Next()
+            NewInputData = self.MC_Next(InputFlow)
             MC_results.append((i, NewInputData, self.Inventory.report(TypeOfPFAS)))
-            if signal:
-                if i in progress_steps:
+            if i in progress_steps:
+                if signal:
                     progress += 5
                     signal.emit(progress)
-                    print('Iteration {} \n ... '.format(i))
                     time.sleep(0.1)
-
+                print('Iteration {} \n ... '.format(i))
         self.Reset_static_Data()
+        if not signal:
+            print(f"Simulation time: {time.time() - start_time:.0f} sec")
+            print(f"Number of iterations: {n}")
         return MC_results
 
     def Reset_static_Data(self):
@@ -241,22 +245,24 @@ class Project():
     def setup_SA(self, InputFlow_object):
         self.InputFlow_object = InputFlow_object
 
-    def SensitivityAnalysis(self, Model, Category, Paramter, Start, Stop, Nstep, TypeOfPFAS='All'):
+    def SensitivityAnalysis(self, Model, Category, Paramter, Start=None,
+                            Stop=None, Nstep=20, TypeOfPFAS='All'):
         try:
             SA_results = []
+            if Model == 'IncomFlow':
+                param_dict = self.InputFlow_object.InputData.Input_dict[Category][Paramter]
+            else:
+                param_dict = self.ProcessModels[Model]['Model'].InputData.Input_dict[Category][Paramter]
+
+            Start = param_dict['minimum'] if Start is None else Start
+            Stop = param_dict['maximum'] if Stop is None else Stop
+            if np.isnan(Start) or np.isnan(Stop):
+                raise ValueError('Set the Start & Stop.')
+
             param_vals = np.linspace(Start, Stop, Nstep)
-            default_value = 'Not Set'
             for i, val in enumerate(param_vals):
                 # Update the Input data
-                if Model == 'IncomFlow':
-                    if default_value == 'Not Set':
-                        default_value = self.InputFlow_object.InputData.Input_dict[Category][Paramter]['amount']
-                    self.InputFlow_object.InputData.Input_dict[Category][Paramter]['amount'] = val
-                else:
-                    if default_value == 'Not Set':
-                        default_value = self.ProcessModels[Model]['Model'].InputData.Input_dict[Category][Paramter]['amount']
-                    self.ProcessModels[Model]['Model'].InputData.Input_dict[Category][Paramter]['amount'] = val
-
+                param_dict['amount'] = val
                 # Calculating results
                 self.Inventory.clear()
                 self.InputFlow_object.calc()
@@ -277,7 +283,7 @@ class Project():
             self.Inventory.clear()
             return(SA_results)
 
-    def plot_network(self, view=True, show_vals=True, all_flow=True):
+    def plot_network(self, view=True, show_vals=True, file_name='Network'):
         """
         To render the generated DOT source code, you also need to install `Graphviz <https://www.graphviz.org/download>`_.
 
@@ -287,7 +293,7 @@ class Project():
         # Set the color for starting waste material
         self._NtwkrNodeColor[self.InputFlow.FlowType] = 'chartreuse2'
         # Initialize PFAS treatment network
-        self.network = graphviz.Digraph(name='Network', filename='Network.gv', format='png', engine='dot')
+        self.network = graphviz.Digraph(name=file_name, filename=file_name + '.gv', format='png', engine='dot')
         self.network.graph_attr['rankdir'] = 'LR'
         self.network.attr(dpi='300')
         self.network.attr(size='5,5')
@@ -295,9 +301,16 @@ class Project():
             self.network.node(x, shape=self._NtwkrNodeShape[x], fillcolor=self._NtwkrNodeColor[x], style='filled', width='1.2')
 
         for e in self._Ntwkedge:
-            self.network.edge(e[0], e[1], label=None, color='black')
+            if show_vals:
+                if e[0] in self.FlowParams:
+                    label = str(self.FlowParams[e[0]][e[1]])
+                else:
+                    label = None
+                self.network.edge(e[0], e[1], label=label, color='black')
+            else:
+                self.network.edge(e[0], e[1], label=None, color='black')
         try:
-            self.network.render('Network', view=view)
+            self.network.render(file_name, view=view)
         except Exception:
             print("""
                   To render the generated DOT source code, you also need to install Graphviz (`Graphviz <https://www.graphviz.org/download>`_).\n
@@ -318,25 +331,25 @@ class Project():
 
         # List of colors: https://flaviocopes.com/rgb-color-codes/
         edge_color_dict = {'FoodWaste': (202, 255, 112),  # darkolivegreen1  #CAFF70
-                           'Compost': (110, 139, 61),  # darkolivegreen4	#6E8B3D
-                           'CompostResiduals': (118, 238, 0),  # chartreuse2	#76EE00
-                           'ADSolids': (124, 252, 0),  # lawn green	#7CFC00
-                           'MSW': (139, 101, 8),  # darkgoldenrod4	#8B6508
-                           'C_DWaste': (238, 232, 205),  # cornsilk2	#EEE8CD
-                           'ADLiquids': (220, 20, 60),  # crimson	#DC143C
-                           'SpentGAC': (189, 183, 107),  # darkkhaki	#BDB76B
-                           'ContaminatedSoil': (128, 138, 135),  # coldgrey	#808A87
-                           'CombustionResiduals': (169, 169, 169),  # darkgray	#A9A9A9
-                           'ContactWater': (142, 229, 238),  # cadetblue2	#8EE5EE
-                           'ContaminatedWater': (0, 0, 139),  # dark blue	#00008B
-                           'LFLeachate': (0, 0, 255),  # blue	#0000FF
-                           'DewateredWWTSolids': (128, 128, 128),  # gray / grey	#808080
-                           'DriedWWTSolids': (192, 192, 192),  # silver	#C0C0C0
-                           'RawWWTSolids': (112, 128, 144),  # slate gray	#708090
-                           'ROConcentrate': (123, 104, 238),  # medium slate blue	#7B68EE
-                           'StabilizedSoil': (72, 61, 139),  # dark slate blue	#483D8B
-                           'ROConc': (70, 130, 180),  # steel blue	#4682B4
-                           'RunOff': (0, 255, 255)}  # aqua	#00FFFF
+                           'Compost': (110, 139, 61),  # darkolivegreen4    #6E8B3D
+                           'CompostResiduals': (118, 238, 0),  # chartreuse2    #76EE00
+                           'ADSolids': (124, 252, 0),  # lawn green    #7CFC00
+                           'MSW': (139, 101, 8),  # darkgoldenrod4    #8B6508
+                           'C_DWaste': (238, 232, 205),  # cornsilk2    #EEE8CD
+                           'ADLiquids': (220, 20, 60),  # crimson    #DC143C
+                           'SpentGAC': (189, 183, 107),  # darkkhaki    #BDB76B
+                           'ContaminatedSoil': (128, 138, 135),  # coldgrey    #808A87
+                           'CombustionResiduals': (169, 169, 169),  # darkgray    #A9A9A9
+                           'ContactWater': (142, 229, 238),  # cadetblue2    #8EE5EE
+                           'ContaminatedWater': (0, 0, 139),  # dark blue    #00008B
+                           'LFLeachate': (0, 0, 255),  # blue    #0000FF
+                           'DewateredWWTSolids': (128, 128, 128),  # gray / grey    #808080
+                           'DriedWWTSolids': (192, 192, 192),  # silver    #C0C0C0
+                           'RawWWTSolids': (112, 128, 144),  # slate gray    #708090
+                           'ROConcentrate': (123, 104, 238),  # medium slate blue    #7B68EE
+                           'StabilizedSoil': (72, 61, 139),  # dark slate blue    #483D8B
+                           'ROConc': (70, 130, 180),  # steel blue    #4682B4
+                           'RunOff': (0, 255, 255)}  # aqua    #00FFFF
 
         for i in Data.loc['Flow_name']:
             if i not in edge_color_dict:
@@ -382,7 +395,7 @@ class Project():
                     thickness=20,
                     line=dict(color="black", width=0.5),
                     label=label,
-                    color='rgba({}, {}, {}, 0.8)'.format(*(176, 196, 222)))  # light steel blue	#B0C4DE
+                    color='rgba({}, {}, {}, 0.8)'.format(*(176, 196, 222)))  # light steel blue    #B0C4DE
 
         link = dict(source=source,
                     target=target,
@@ -399,7 +412,7 @@ class Project():
                          node=node,
                          link=link)
         fig = go.Figure(data=[data], layout=layout)
-        plot(fig, filename=filename if filename else 'sankey.html', auto_open=view)
+        plot(fig, filename=filename + '.html' if filename else 'sankey.html', auto_open=view)
 
         # Store data for ploting the sankey
         store_data = {}
@@ -411,5 +424,5 @@ class Project():
         store_data['node'] = node
         store_data['link'] = link
 
-        with open('Sankey_Data.JSON', 'w') as outfile:
+        with open(filename + '.JSON' if filename else 'Sankey_Data.JSON', 'w') as outfile:
             json.dump(store_data, outfile, indent=4, cls=NpEncoder)
