@@ -27,7 +27,7 @@ def test_project():
     demo.Inventory.Inv
 
     demo.setup_MC(InputFlow)
-    demo.MC_Next()
+    demo.MC_Next(True)
     demo.Inventory.Inv
 
     results = demo.MC_Run(100)
